# LoRaWANデバイスの位置情報可視化

本プロジェクトは、2022年10月にリリースされた builders.flash の記事「LoRaWANで家から子供の初めてのおつかいを見守ろう」で使用します。
Semtech LR1110 の位置情報をAWS上に保存し、ブラウザベースの地図アプリケーションにて現在位置を表示します。
また、GeoFencingの設定をおこなうことで、デバイスが指定した範囲を外れたときに、予め指定した電話番号に電話発信をおこないます。
使用するLoRaWAN対応デバイスは、立花電子ソリューションズ様のZeroCarbon評価ボードです。

# アーキテクチャ
![image](/doc/image/lorawan_demo_architecture_ver3.jpg) 
## IoT part
![image](/doc/arch.dio.png) 

## LoRa Cloud の連携

lib/loracloud-stack.ts では、AWS CDKを用いて次のことを実現するAWS環境を構築します。
* ZeroCarbonのデータフォーマットをサポートし、デバイスから分割されて送られるGNSSデータもしくはWi-Fiデータを、一時的にDynamoDBにキャッシュする
* すべてのデータが到着したらそれらを結合し LoRa Cloud に緯度経度を問い合わせる
* 得られた結果を、Amazon Location ServiceのTracker情報として登録する

### AWS CDK
環境構築にはCDKを用いています。

* AWS CDKについては、[こちら](https://aws.amazon.com/jp/cdk/)をご覧ください。また、[こちら](https://cdkworkshop.com/) のWorkshopにより、使い方の習得が可能です。
### Lambda
* lambda/lora-cloud.ts に保存されています。テストコードは、test/lambda/lora-cloud.test.ts にあります。
このプロジェクトでは、緯度経度取得に関する情報のみをハンドリングします。ZeroCarbon データフォーマットでは、温湿度情報などもアップロードされますが、現時点ではサポートしていません。
* LoRa Cloudの認証情報は、Secrets Manager を使って設定・取得します。

### dynamoDB
ペイロードの一時キャッシュとして使用しています。連続して送られてくるデータの一時保管の用途のため、TTLを1分で設定しています。WiFi、GNSSそれぞれ同じTableで管理しています。

### IoT Core
IoT ルールの作成を行っています。 `dt/lorawan` に届いたメッセージを受けて、前述のLambdaをInvokeします。

# デプロイ
## LoRa Cloudの連携
Clone したリポジトリのルートで以下の処理をおこないます。
  
```
cdk bootstrap
cdk deploy --all
```

[名前付きプロファイル](https://docs.aws.amazon.com/ja_jp/cli/latest/userguide/cli-configure-profiles.html)を設定し、デプロイのミスを防ぐことを推奨します。
デプロイが完了したら、LoRa Cloud Token を Secrets Manager に設定してください。

# 使い方

## デバイスの準備
* LoRaWANデバイス
  * このプロジェクトでは、Semtech LR1110を搭載する、ZeroCarbon LoRa EvaluationBoard を LoRaWANデバイスとして用います。
* このデバイスの詳細、購入方法については、[立花電子ソリューションズ様 ](https://tachibana-denshi-solutions.co.jp/lora_about.php) サイトにてご確認頂けます。
* LoRaWAN ゲートウェイ
  * [AWS IoT Core for LoRaWAN に対応するゲートウェイ](https://devices.amazonaws.com/)からお選びください。



