import * as AWS from "aws-sdk";
const REGION = process.env.REGION!;
const location = new AWS.Location({ region: REGION });
import { Handler } from "aws-lambda";

export const sendLocation: Handler = async (event) => {
  //console.log(JSON.stringify(event));
  await sendLocationInternal(event.deviceId, event.longitude, event.latitude);
};

async function sendLocationInternal(
  deviceId: string,
  longitude: number | null,
  latitude: number | null,
) {
  try {
    await updateDevicePosition(deviceId, longitude, latitude);
  } catch (e) {
    console.log(e);
    if (e instanceof Error && e.name === "ResourceNotFoundException") {
      await createTracker(deviceId);
      await updateDevicePosition(deviceId, longitude, latitude);
    }
  }
}

async function updateDevicePosition(
  trackerName: string,
  longitude: number | null,
  latitude: number | null,
) {
  await new Promise<void>((resolve, reject) => {
    location.batchUpdateDevicePosition(
      {
        TrackerName: trackerName,
        Updates: [
          {
            DeviceId: trackerName,
            Position: [longitude!, latitude!],
            SampleTime: new Date(),
          },
        ],
      },
      (err, data) => {
        if (err) {
          reject(err);
          return;
        }
        console.log(`${err} ${JSON.stringify(data)}`);
        resolve();
      },
    );
  });
}

async function createTracker(trackerName: string) {
  await new Promise<void>((resolve, reject) => {
    location.createTracker(
      {
        TrackerName: trackerName,
      },
      (err, data) => {
        if (err) {
          reject(err);
          return;
        }
        console.log(`${err} ${data}`);
        resolve();
      },
    );
  });
}
