import { Stack, StackProps } from "aws-cdk-lib";
import { Construct } from "constructs";
import { NodejsFunction } from "aws-cdk-lib/aws-lambda-nodejs";
import * as iot from "@aws-cdk/aws-iot-alpha";
import * as actions from "@aws-cdk/aws-iot-actions-alpha";
import * as iam from "aws-cdk-lib/aws-iam";

export class LocationStack extends Stack {
  constructor(scope: Construct, id: string, props?: StackProps) {
    super(scope, id, props);
    const region_account = `${process.env.CDK_DEFAULT_REGION}:${process.env.CDK_DEFAULT_ACCOUNT}:`;

    const sendLocationFunction = new NodejsFunction(
      this,
      `LoRaCloudPayloadHandler_sendLocation`,
      {
        entry: "lambda/location.ts",
        handler: "sendLocation",
      },
    );
    sendLocationFunction.addToRolePolicy(
      new iam.PolicyStatement({
        effect: iam.Effect.ALLOW,
        resources: [`arn:aws:geo:${region_account}tracker/*`],
        actions: ["geo:BatchUpdateDevicePosition", "geo:CreateTracker"],
      }),
    );

    new iot.TopicRule(this, "LocationTopicRule", {
      sql: iot.IotSql.fromStringAsVer20160323(
        `SELECT *, topic(2) as deviceId, timestamp() as timestamp FROM 'dt/+/location'`,
      ),
      actions: [new actions.LambdaFunctionAction(sendLocationFunction)],
    });
  }
}
