import * as dotenv from "dotenv";
dotenv.config();

import * as loraCloud from "../../lambda/lora-cloud";
import * as location from "../../lambda/location";

test("getLocation", async () => {
  const devEui = "70b3d57ed0050033";
  const hex =
    "016d7c821ca9aa086ad70358523937d68b0060a88ef1a0c43eb35175ac98123f4a848aeaaa42ccc04a55b52f8c8af808";
  const [lat, lng] = await loraCloud.getLocation(hex, devEui, 1318569713.0);
  console.log(`location: ${lat}, ${lng}`);
});

test("addPayload", async () => {
  await loraCloud.addPayloadInternal(sample);
  await loraCloud.addPayloadInternal(sample2);
});

test("addPayloadGnss", async () => {
  await loraCloud.addPayloadInternal(sampleGnss1);
  await loraCloud.addPayloadInternal(sampleGnss2);
});

test("addPayloadWiFi", async () => {
  await loraCloud.addPayloadInternal(sampleWifi1);
  await loraCloud.addPayloadInternal(sampleWifi2);
});

test("dynamo", async () => {
  const deviceName = "502aceba-ec26-4983-9402-d3174048ba90";
  await loraCloud.updateItem(deviceName, 50, -1, "", -1, []);
  let data = await loraCloud.getItemFromTable(deviceName);
  expect(data.gnssDataLength).toBe(50);
  await loraCloud.updateItem(deviceName, 50, data.counter + 1, "abc", -1, []);
  data = await loraCloud.getItemFromTable(deviceName);
  await loraCloud.updateItem(
    deviceName,
    50,
    data.counter + 1,
    data.gnssData + "def",
    -1,
    [],
  );
  data = await loraCloud.getItemFromTable(deviceName);
  expect(data.gnssData).toBe("abcdef");

  await loraCloud.updateItem(deviceName, -1, -1, "", -1, []);

  await loraCloud.updateItem(deviceName, -1, 1, "", 1, [1, 2, 3, 4, 5]);
  data = await loraCloud.getItemFromTable(deviceName);
  expect(data.wifiData).toStrictEqual([1, 2, 3, 4, 5]);

  await loraCloud.updateItem(deviceName, -1, -1, "", -1, []);
});

test("parser", () => {
  let data = loraCloud.parsePayload(sample);
  // console.log(data);

  // payload is empty.
  try {
    loraCloud.parsePayload(sample2);
  } catch (e) {
    // console.error(e);
  }

  data = loraCloud.parsePayload(sampleGnss1);
  // console.log(data);
  expect(data.dataType).toBe("gnss");

  data = loraCloud.parsePayload(sampleGnss2);
  // console.log(data);

  data = loraCloud.parsePayload(sampleWifi1);
  //console.log(data);
  expect(data.dataType).toBe("wifi");

  data = loraCloud.parsePayload(sampleWifi2);
  //console.log(data);
});

const sample2 = {
  WirelessDeviceId: "502aceba-ec26-4983-9402-d3174048ba90",
  PayloadData: "",
  WirelessMetadata: {
    LoRaWAN: {
      DevEui: "70b3d57ed004f46c",
      FPort: 2,
    },
  },
};

// 47 01 40 47 32 00 00 00 00 00 00 00 00 00 00 00 11 11 11 11 11 11 11 11 11 11 22 22 22 22 22 22 22 22 22 22 33 33 33 33 33 33 33 33 33 33 44 44 44
// RwFARzIAAAAAAAAAAAAAABEREREREREREREiIiIiIiIiIiIiMzMzMzMzMzMzM0RERA==
const sampleGnss1 = {
  WirelessDeviceId: "502aceba-ec26-4983-9402-d3174048ba90",
  PayloadData:
    "RwFARzIAAAAAAAAAAAAAABEREREREREREREiIiIiIiIiIiIiMzMzMzMzMzMzM0RERA==",
  WirelessMetadata: {
    LoRaWAN: {
      DevEui: "70b3d57ed004f46c",
      FPort: 2,
    },
  },
};
// 47 00 44 44 44 44 44 44 44 55 55 55 55 55 55 55 55 55 55 55 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
// RwBEREREREREVVVVVVVVVVVVVVUAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA==
const sampleGnss2 = {
  WirelessDeviceId: "502aceba-ec26-4983-9402-d3174048ba90",
  PayloadData:
    "RwBEREREREREVVVVVVVVVVVVVVUAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA==",
  WirelessMetadata: {
    LoRaWAN: {
      DevEui: "70b3d57ed004f46c",
      FPort: 2,
    },
  },
};
const sample = {
  WirelessDeviceId: "502aceba-ec26-4983-9402-d3174048ba90",
  PayloadData: "U/wIXAv8FwAAAAA=",
  WirelessMetadata: {
    LoRaWAN: {
      ADR: true,
      Bandwidth: 125,
      ClassB: false,
      CodeRate: "4/5",
      DataRate: "2",
      DevAddr: "01e2b664",
      DevEui: "70b3d57ed004f46c",
      FCnt: 1,
      FOptLen: 0,
      FPort: 2,
      Frequency: "922600000",
      Gateways: [
        {
          GatewayEui: "647fdafffe00b22f",
          Rssi: -32,
          Snr: 13,
        },
      ],
      MIC: "c1b3436c",
      MType: "UnconfirmedDataUp",
      Major: "LoRaWANR1",
      Modulation: "LORA",
      PolarizationInversion: false,
      SpreadingFactor: 10,
      Timestamp: "2022-08-09T06:18:02Z",
    },
  },
};

// wifi
// VwFAVwAEWawBAIUXAAAbwgsAAAAAALCVdeg2xgYBrraVdeg2xgYBr+hs80W7LQsBpw==
// 57 01 40 57 00 04 59 ac 01 00 85 17 00 00 1b c2 0b 00 00 00 00 00 b0 95 75 e8 36 c6 06 01 ae b6 95 75 e8 36 c6 06 01 af e8 6c f3 45 bb 2d 0b 01 a7
const sampleWifi1 = {
  WirelessDeviceId: "502aceba-ec26-4983-9402-d3174048ba90",
  PayloadData:
    "VwFAVwAEWawBAIUXAAAbwgsAAAAAALCVdeg2xgYBrraVdeg2xgYBr+hs80W7LQsBpw==",
  WirelessMetadata: {
    LoRaWAN: {
      DevEui: "70b3d57ed004f46c",
      FPort: 2,
    },
  },
};
// VwDYD5nkJzYLAacAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA==
// 57 00 d8 0f 99 e4 27 36 0b 01 a7 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
const sampleWifi2 = {
  WirelessDeviceId: "502aceba-ec26-4983-9402-d3174048ba90",
  PayloadData:
    "VwDYD5nkJzYLAacAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA==",
  WirelessMetadata: {
    LoRaWAN: {
      DevEui: "70b3d57ed004f46c",
      FPort: 2,
    },
  },
};
/*
test("SSM", async () => {
  const token = await loraCloud.getLoRaCloudTokenFromSM(
    "prototyping/lorawan/loracloud",
  );
  expect(token).toBe("please replace here.");
});
*/
